@@ -1,4 +1,3 @@
-# coding: utf8
 """
     weasyprint.formatting_structure.boxes
     -------------------------------------
@@ -52,23 +51,14 @@
 
     See respective docstrings for details.
 
-    :copyright: Copyright 2011-2014 Simon Sapin and contributors, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-
 """
 
-from __future__ import division, unicode_literals
-
 import itertools
 
-from ..compat import xrange
-from ..css.computed_values import ZERO_PIXELS
+from ..css import computed_from_cascaded
 
 
-# The *Box classes have many attributes and methods, but that's the way it is
-# pylint: disable=R0904,R0902
-
-class Box(object):
+class Box:
     """Abstract base class for all boxes."""
     # Definitions for the rules generating anonymous table boxes
     # http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
@@ -76,60 +66,69 @@ class Box(object):
     internal_table_or_caption = False
     tabular_container = False
 
+    # Keep track of removed collapsing spaces for wrap opportunities.
+    leading_collapsible_space = False
+    trailing_collapsible_space = False
+
     # Default, may be overriden on instances.
     is_table_wrapper = False
+    is_flex_item = False
     is_for_root_element = False
-    bookmark_label = None
-    bookmark_level = None
+    is_column = False
+
+    # Other properties
     transformation_matrix = None
+    bookmark_label = None
+    string_set = None
 
     # Default, overriden on some subclasses
     def all_children(self):
         return ()
 
-    def __init__(self, element_tag, sourceline, style):
+    def __init__(self, element_tag, style, element):
         self.element_tag = element_tag
-        self.sourceline = sourceline  # for debugging only
-        # Copying might not be needed, but let’s be careful with mutable
-        # objects.
-        self.style = style.copy()
-        bookmark_level = style.bookmark_level
-        if bookmark_level != 'none':
-            self.bookmark_level = bookmark_level
+        self.element = element
+        self.style = style
+        self.remove_decoration_sides = set()
 
     def __repr__(self):
-        return '<%s %s %s>' % (
-            type(self).__name__, self.element_tag, self.sourceline)
+        return '<%s %s>' % (type(self).__name__, self.element_tag)
 
     @classmethod
     def anonymous_from(cls, parent, *args, **kwargs):
         """Return an anonymous box that inherits from ``parent``."""
-        return cls(parent.element_tag, parent.sourceline,
-                   parent.style.inherit_from(),
-                   *args, **kwargs)
+        style = computed_from_cascaded(
+            cascaded={}, parent_style=parent.style, element=None)
+        return cls(parent.element_tag, style, parent.element, *args, **kwargs)
 
     def copy(self):
         """Return shallow copy of the box."""
         cls = type(self)
-        # Create a new instance without calling __init__: initializing
-        # styles may be kinda expensive, no need to do it again.
+        # Create a new instance without calling __init__: parameters are
+        # different depending on the class.
         new_box = cls.__new__(cls)
         # Copy attributes
         new_box.__dict__.update(self.__dict__)
-        new_box.style = self.style.copy()
         return new_box
 
-    def translate(self, dx=0, dy=0):
+    def deepcopy(self):
+        """Return a copy of the box with recursive copies of its children."""
+        return self.copy()
+
+    def translate(self, dx=0, dy=0, ignore_floats=False):
         """Change the box’s position.
 
         Also update the children’s positions accordingly.
 
         """
         # Overridden in ParentBox to also translate children, if any.
+        if dx == 0 and dy == 0:
+            return
         self.position_x += dx
         self.position_y += dy
         for child in self.all_children():
-            child.translate(dx, dy)
+            if not (ignore_floats and child.is_floated()):
+                child.translate(dx, dy, ignore_floats)
 
     # Heights and widths
 
@@ -271,62 +270,69 @@ class Box(object):
 
     def is_floated(self):
         """Return whether this box is floated."""
-        return self.style.float != 'none'
+        return self.style['float'] != 'none'
 
     def is_absolutely_positioned(self):
         """Return whether this box is in the absolute positioning scheme."""
-        return self.style.position in ('absolute', 'fixed')
+        return self.style['position'] in ('absolute', 'fixed')
+
+    def is_running(self):
+        """Return whether this box is a running element."""
+        return self.style['position'][0] == 'running()'
 
     def is_in_normal_flow(self):
         """Return whether this box is in normal flow."""
-        return not (self.is_floated() or self.is_absolutely_positioned())
+        return not (
+            self.is_floated() or self.is_absolutely_positioned() or
+            self.is_running())
+
+    # Start and end page values for named pages
+
+    def page_values(self):
+        """Return start and end page values."""
+        return (self.style['page'], self.style['page'])
 
 
 class ParentBox(Box):
     """A box that has children."""
-    def __init__(self, element_tag, sourceline, style, children):
-        super(ParentBox, self).__init__(element_tag, sourceline, style)
+    def __init__(self, element_tag, style, element, children):
+        super().__init__(element_tag, style, element)
         self.children = tuple(children)
 
     def all_children(self):
         return self.children
 
-    def enumerate_skip(self, skip_num=0):
-        """Yield ``(child, child_index)`` tuples for each child.
-
-        ``skip_num`` children are skipped before iterating over them.
-
-        """
-        for index in xrange(skip_num, len(self.children)):
-            yield index, self.children[index]
-
     def _reset_spacing(self, side):
         """Set to 0 the margin, padding and border of ``side``."""
+        self.remove_decoration_sides.add(side)
         setattr(self, 'margin_%s' % side, 0)
         setattr(self, 'padding_%s' % side, 0)
         setattr(self, 'border_%s_width' % side, 0)
 
-        self.style['margin_%s' % side] = ZERO_PIXELS
-        self.style['padding_%s' % side] = ZERO_PIXELS
-        self.style['border_%s_width' % side] = 0
-
-    def _remove_decoration(self, start, end):
+    def remove_decoration(self, start, end):
+        if self.style['box_decoration_break'] == 'clone':
+            return
         if start:
             self._reset_spacing('top')
         if end:
             self._reset_spacing('bottom')
 
-    def copy_with_children(self, new_children, is_start=True, is_end=True):
+    def copy_with_children(self, new_children):
         """Create a new equivalent box with given ``new_children``."""
         new_box = self.copy()
         new_box.children = tuple(new_children)
-        if not is_start:
-            new_box.outside_list_marker = None
-            if new_box.bookmark_level:
-                del new_box.bookmark_level
-        new_box._remove_decoration(not is_start, not is_end)
+
+        # Clear and reset removed decorations as we don't want to keep the
+        # previous data, for example when a box is split between two pages.
+        self.remove_decoration_sides = set()
+
         return new_box
 
+    def deepcopy(self):
+        result = self.copy()
+        result.children = tuple(child.deepcopy() for child in self.children)
+        return result
+
     def descendants(self):
         """A flat generator for a box, its children and descendants."""
         yield self
@@ -346,6 +352,19 @@ class ParentBox(Box):
             else:  # pragma: no cover
                 raise ValueError('Table wrapper without a table')
 
+    def page_values(self):
+        start_value, end_value = super().page_values()
+        if self.children:
+            if len(self.children) == 1:
+                page_values = self.children[0].page_values()
+                start_value = page_values[0] or start_value
+                end_value = page_values[1] or end_value
+            else:
+                start_box, end_box = self.children[0], self.children[-1]
+                start_value = start_box.page_values()[0] or start_value
+                end_value = end_box.page_values()[1] or end_value
+        return start_value, end_value
+
 
 class BlockLevelBox(Box):
     """A box that participates in an block formatting context.
@@ -377,11 +396,6 @@ class BlockBox(BlockContainerBox, BlockLevelBox):
     generates a block box.
 
     """
-    # TODO: remove this when outside list marker are absolute children
-    def all_children(self):
-        marker = getattr(self, 'outside_list_marker', None)
-        return (itertools.chain(self.children, [marker])
-                if marker else self.children)
 
 
 class LineBox(ParentBox):
@@ -394,9 +408,15 @@ class LineBox(ParentBox):
     be split into multiple line boxes, one for each actual line.
 
     """
-    def __init__(self, element_tag, sourceline, style, children):
-        assert style.anonymous
-        super(LineBox, self).__init__(element_tag, sourceline, style, children)
+    text_overflow = 'clip'
+    block_ellipsis = 'none'
+
+    @classmethod
+    def anonymous_from(cls, parent, *args, **kwargs):
+        box = super().anonymous_from(parent, *args, **kwargs)
+        if parent.style['overflow'] != 'visible':
+            box.text_overflow = parent.style['text_overflow']
+        return box
 
 
 class InlineLevelBox(Box):
@@ -409,8 +429,10 @@ class InlineLevelBox(Box):
     ``inline-block`` generates an inline-level box.
 
     """
-    def _remove_decoration(self, start, end):
-        ltr = self.style.direction == 'ltr'
+    def remove_decoration(self, start, end):
+        if self.style['box_decoration_break'] == 'clone':
+            return
+        ltr = self.style['direction'] == 'ltr'
         if start:
             self._reset_spacing('left' if ltr else 'right')
         if end:
@@ -441,19 +463,25 @@ class TextBox(InlineLevelBox):
     inline boxes" are also text boxes.
 
     """
-    def __init__(self, element_tag, sourceline, style, text):
-        assert style.anonymous
+    justification_spacing = 0
+
+    # http://stackoverflow.com/questions/16317534/
+    ascii_to_wide = dict((i, chr(i + 0xfee0)) for i in range(0x21, 0x7f))
+    ascii_to_wide.update({0x20: '\u3000', 0x2D: '\u2212'})
+
+    def __init__(self, element_tag, style, element, text):
         assert text
-        super(TextBox, self).__init__(element_tag, sourceline, style)
-        text_transform = style.text_transform
+        super().__init__(element_tag, style, element)
+        text_transform = style['text_transform']
         if text_transform != 'none':
             text = {
                 'uppercase': lambda t: t.upper(),
                 'lowercase': lambda t: t.lower(),
                 # Python’s unicode.captitalize is not the same.
                 'capitalize': lambda t: t.title(),
+                'full-width': lambda t: t.translate(self.ascii_to_wide),
             }[text_transform](text)
-        if style.hyphens == 'none':
+        if style['hyphens'] == 'none':
             text = text.replace('\u00AD', '')  # U+00AD SOFT HYPHEN (SHY)
         self.text = text
 
@@ -491,8 +519,8 @@ class ReplacedBox(Box):
     and is opaque from CSS’s point of view.
 
     """
-    def __init__(self, element_tag, sourceline, style, replacement):
-        super(ReplacedBox, self).__init__(element_tag, sourceline, style)
+    def __init__(self, element_tag, style, element, replacement):
+        super().__init__(element_tag, style, element)
         self.replacement = replacement
 
 
@@ -524,10 +552,15 @@ class TableBox(BlockLevelBox, ParentBox):
     def all_children(self):
         return itertools.chain(self.children, self.column_groups)
 
-    def translate(self, dx=0, dy=0):
+    def translate(self, dx=0, dy=0, ignore_floats=False):
+        if dx == 0 and dy == 0:
+            return
         self.column_positions = [
             position + dx for position in self.column_positions]
-        return super(TableBox, self).translate(dx, dy)
+        return super().translate(dx, dy, ignore_floats)
+
+    def page_values(self):
+        return (self.style['page'], self.style['page'])
 
 
 class InlineTableBox(TableBox):
@@ -574,6 +607,11 @@ class TableColumnGroupBox(ParentBox):
     padding_left = 0
     padding_right = 0
 
+    def get_cells(self):
+        """Return cells that originate in the group's columns."""
+        return [
+            cell for column in self.children for cell in column.get_cells()]
+
 
 # Not really a parent box, but pretending to be removes some corner cases.
 class TableColumnBox(ParentBox):
@@ -596,6 +634,14 @@ class TableColumnBox(ParentBox):
     padding_left = 0
     padding_right = 0
 
+    def get_cells(self):
+        """Return cells that originate in the column.
+
+        May be overriden on instances.
+
+        """
+        return []
+
 
 class TableCellBox(BlockContainerBox):
     """Box for elements with ``display: table-cell``"""
@@ -623,8 +669,8 @@ class PageBox(ParentBox):
     def __init__(self, page_type, style):
         self.page_type = page_type
         # Page boxes are not linked to any element.
-        super(PageBox, self).__init__(
-            element_tag=None, sourceline=None, style=style, children=[])
+        super().__init__(
+            element_tag=None, style=style, element=None, children=[])
 
     def __repr__(self):
         return '<%s %s>' % (type(self).__name__, self.page_type)
@@ -632,11 +678,31 @@ class PageBox(ParentBox):
 
 class MarginBox(BlockContainerBox):
     """Box in page margins, as defined in CSS3 Paged Media"""
-    def __init__(self, at_keyword, style, children=[]):
+    def __init__(self, at_keyword, style):
         self.at_keyword = at_keyword
         # Margin boxes are not linked to any element.
-        super(MarginBox, self).__init__(
-            element_tag=None, sourceline=None, style=style, children=children)
+        super().__init__(
+            element_tag=None, style=style, element=None, children=[])
 
     def __repr__(self):
         return '<%s %s>' % (type(self).__name__, self.at_keyword)
+
+
+class FlexContainerBox(ParentBox):
+    """A box that contains only flex-items."""
+
+
+class FlexBox(FlexContainerBox, BlockLevelBox):
+    """A box that is both block-level and a flex container.
+
+    It behaves as block on the outside and as a flex container on the inside.
+
+    """
+
+
+class InlineFlexBox(FlexContainerBox, InlineLevelBox):
+    """A box that is both inline-level and a flex container.
+
+    It behaves as inline on the outside and as a flex container on the inside.
+
+    """
